@@ -27,16 +27,17 @@
 #' `timestamp` and `cumtime` are also updated accordingly.
 #' @export
 #' @examples
-#' library(dplyr)
+#' library(magrittr)
 #' poa <- read_gtfs(system.file("extdata/poa.zip", package="gtfs2gps")) %>%
-#'   filter_week_days() %>%
+#'   gtfstools::filter_by_shape_id("T2-1") %>%
+#'   gtfstools::filter_by_weekday(c("monday", "wednesday")) %>%
 #'   filter_single_trip()
 #'
 #' poa_gps <- gtfs2gps(poa)
 #' poa_gps_new <- adjust_speed(poa_gps)
 adjust_speed <- function(gps_data, min_speed = 2, max_speed = 80, new_speed = NULL, clone = TRUE){
   if(clone) gps_data <- data.table::copy(gps_data)
-
+  
   max_speed <- units::set_units(max_speed, "km/h") %>% 
     units::set_units("m/s") %>%
     units::drop_units()
@@ -44,24 +45,34 @@ adjust_speed <- function(gps_data, min_speed = 2, max_speed = 80, new_speed = NU
   min_speed <- units::set_units(min_speed, "km/h") %>%
     units::set_units("m/s") %>%
     units::drop_units()
-
+  
   gps_data[, speed := units::drop_units(units::set_units(speed, "m/s"))]
   gps_data[speed == "Inf" | is.na(speed) | is.nan(speed), speed := NA]
-  gps_data[!data.table::between(x = speed, lower = min_speed, upper = max_speed), speed := NA]
+  gps_data[,stopped_bus := data.table::fifelse(!is.na(stop_sequence) 
+                                               & round(as.numeric(dist)) == 0
+                                               & !is.na(speed),
+                                               TRUE,
+                                               FALSE)]
+  gps_data[!stopped_bus & 
+             !data.table::between(x = speed
+                                  , lower = min_speed
+                                  , upper = max_speed), speed := NA]
   
   if(is.null(new_speed))
-    new_speed <- mean(gps_data$speed, na.rm = TRUE)
+    new_speed <- weighted.mean(x = gps_data[stopped_bus == FALSE,speed]
+                               ,w = gps_data[stopped_bus == FALSE,as.numeric(dist)]
+                               ,na.rm=TRUE)
   else
     new_speed <- units::set_units(new_speed, "km/h") %>%
-      units::set_units("m/s") %>%
-      units::drop_units()
-
+    units::set_units("m/s") %>%
+    units::drop_units()
+  
   gps_data[, dist := units::drop_units(dist)]
   gps_data[, cumdist := units::drop_units(cumdist)]
   gps_data[, cumtime := units::drop_units(cumtime)]
   
-  gps_data[is.na(speed), speed := new_speed]
-
+  gps_data[is.na(speed) & stopped_bus == FALSE, speed := new_speed]
+  
   gps_data[, time := dist / speed]
   gps_data[, cumtime := cumsum(time), by = .(shape_id, trip_id, trip_number)]
   
@@ -69,8 +80,8 @@ adjust_speed <- function(gps_data, min_speed = 2, max_speed = 80, new_speed = NU
     tmp_gps_data <- gps_data[shape_id == i, ]
     
     if(nrow(tmp_gps_data[!is.na(timestamp)]) < 2){
-     message(paste0("timestamp column of shape_id ", i, " was not adjusted as there are less than two timestamp's."))
-     return(tmp_gps_data)
+      message(paste0("timestamp column of shape_id ", i, " was not adjusted as there are less than two timestamp's."))
+      return(tmp_gps_data)
     }
     
     id_first <- which(!is.na(tmp_gps_data$timestamp))[1]
@@ -81,18 +92,18 @@ adjust_speed <- function(gps_data, min_speed = 2, max_speed = 80, new_speed = NU
     }
     else if(id_first == 1){
       tmp_gps_data[, 
-        timestamp := round(timestamp[1] + cumtime),
-        by = .(trip_id, trip_number)]
+                   timestamp := round(timestamp[1] + cumtime),
+                   by = .(trip_id, trip_number)]
     }else{
       # before the first valid observation
       tmp_gps_data[1:id_first,
-        timestamp := round(timestamp[id_first] - cumtime),
-        by = .(trip_id, trip_number)]
-     
+                   timestamp := round(timestamp[id_first] - cumtime),
+                   by = .(trip_id, trip_number)]
+      
       # after the first valid observation
       tmp_gps_data[id_first:.N,
-        timestamp := round(timestamp[id_first] + cumtime),
-        by = .(trip_id, trip_number)]
+                   timestamp := round(timestamp[id_first] + cumtime),
+                   by = .(trip_id, trip_number)]
     }
     
     # midnight trips fix
@@ -100,14 +111,15 @@ adjust_speed <- function(gps_data, min_speed = 2, max_speed = 80, new_speed = NU
     
     return(tmp_gps_data)
   }) %>% data.table::rbindlist()
-
+  
   gps_data[, speed := units::set_units(units::set_units(speed, "m/s"), "km/h")]
   gps_data[, dist := units::set_units(dist, "m")]
   gps_data[, cumdist := units::set_units(cumdist, "m")]
   gps_data[, time := units::set_units(time, "s")]
   gps_data[, cumtime := units::set_units(cumtime, "s")]
-
+  
   gps_data[, timestamp := data.table::as.ITime(timestamp)]
-
+  gps_data[,stopped_bus := NULL]
+  
   return(gps_data)
 }
